@@ -303,7 +303,7 @@ def fit_numeric_outcome_treatment(
     n = X.shape[0]
     all_bad = []
     for vi in var_list:
-        n_bad = sum(vtreat.util.is_bad(X[vi]))
+        n_bad = numpy.sum(vtreat.util.is_bad(X[vi]))
         if n_bad >= n:
             all_bad = all_bad + [vi]
         if (n_bad > 0) and (n_bad < n):
@@ -370,6 +370,8 @@ def fit_numeric_outcome_treatment(
     return {
         "outcome_name": outcome_name,
         "cols_to_copy": cols_to_copy,
+        "num_list": num_list,
+        "cat_list": cat_list,
         "xforms": xforms,
     }
 
@@ -390,7 +392,7 @@ def fit_binomial_outcome_treatment(
     n = X.shape[0]
     all_bad = []
     for vi in var_list:
-        n_bad = sum(vtreat.util.is_bad(X[vi]))
+        n_bad = numpy.sum(vtreat.util.is_bad(X[vi]))
         if n_bad >= n:
             all_bad = all_bad + [vi]
         if (n_bad > 0) and (n_bad < n):
@@ -448,6 +450,8 @@ def fit_binomial_outcome_treatment(
     return {
         "outcome_name": outcome_name,
         "cols_to_copy": cols_to_copy,
+        "num_list": num_list,
+        "cat_list": cat_list,
         "xforms": xforms,
     }
 
@@ -468,7 +472,7 @@ def fit_multinomial_outcome_treatment(
     n = X.shape[0]
     all_bad = []
     for vi in var_list:
-        n_bad = sum(vtreat.util.is_bad(X[vi]))
+        n_bad = numpy.sum(vtreat.util.is_bad(X[vi]))
         if n_bad >= n:
             all_bad = all_bad + [vi]
         if (n_bad > 0) and (n_bad < n):
@@ -533,6 +537,8 @@ def fit_multinomial_outcome_treatment(
     return {
         "outcome_name": outcome_name,
         "cols_to_copy": cols_to_copy,
+        "num_list": num_list,
+        "cat_list": cat_list,
         "xforms": xforms,
     }
 
@@ -551,7 +557,7 @@ def fit_unsupervised_treatment(*, X, var_list, outcome_name, cols_to_copy, param
     n = X.shape[0]
     all_bad = []
     for vi in var_list:
-        n_bad = sum(vtreat.util.is_bad(X[vi]))
+        n_bad = numpy.sum(vtreat.util.is_bad(X[vi]))
         if n_bad >= n:
             all_bad = all_bad + [vi]
         if (n_bad > 0) and (n_bad < n):
@@ -597,11 +603,13 @@ def fit_unsupervised_treatment(*, X, var_list, outcome_name, cols_to_copy, param
     return {
         "outcome_name": outcome_name,
         "cols_to_copy": cols_to_copy,
+        "num_list": num_list,
+        "cat_list": cat_list,
         "xforms": xforms,
     }
 
 
-def pre_prep_frame(x, *, col_list, cols_to_copy):
+def pre_prep_frame(x, *, col_list, cols_to_copy, cat_cols=None):
     """Create a copy of pandas.DataFrame x restricted to col_list union cols_to_copy with col_list - cols_to_copy
     converted to only string and numeric types.  New pandas.DataFrame has trivial indexing.  If col_list
     is empty it is interpreted as all columns."""
@@ -624,16 +632,23 @@ def pre_prep_frame(x, *, col_list, cols_to_copy):
         raise ValueError("no variables")
     x = x.loc[:, col_list]
     x = x.reset_index(inplace=False, drop=True)
+    cat_col_set = None
+    if cat_cols is not None:
+        cat_col_set = set(cat_cols)
     for c in x.columns:
         if c in cset:
             continue
         bad_ind = vtreat.util.is_bad(x[c])
-        if vtreat.util.can_convert_v_to_numeric(x[c]):
+        if cat_col_set is not None:
+            numeric_col = c not in cat_col_set
+        else:
+            numeric_col = vtreat.util.can_convert_v_to_numeric(x[c])
+        if numeric_col:
             x[c] = vtreat.util.safe_to_numeric_array(x[c])
         else:
             # https://stackoverflow.com/questions/22231592/pandas-change-data-type-of-series-to-string
-            x[c] = numpy.asarray(x[c].apply(str), dtype=str)
-        x.loc[bad_ind, c] = numpy.nan
+            x[c] = x[c].astype(str)
+        x.loc[bad_ind, c] = None
     return x
 
 
